# 4part-random.py written for Python 2.7
#
# This Python program reads a file describing a graph. Then it
#randomly chooses nodes with probability 0.5 to assign functions to.
#The function assigned to one of the chosen nodes, is  chosen uniformly at random from four functions.
# Then it creates the concrete ILP for the resulting max 4-cut problem. 
# DG October 18, 2017

# call on a command line in a terminal window as:
# python 4part-random.py graph-file ilpfile.lp number-of-nodes

# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

import random

arg1 = sys.argv[1]    # assign the graph-file name to the variable arg1
arg2 = sys.argv[2]    # assign the name of the ILP file to the variable arg2
n = sys.argv[3]  # assign the number of nodes to n
                 # The values in arg1, arg2, and n are strings.
n = int(n)       # and convert the number from a string to an integer

objective = ""
constraints = ""
binaries = ""

has_feature=[]  # has_feature is a list that records, at position i, whether node i has a feature (= 1) or not (= 0).
feature=[]      # feature is a list that records, at position i, which feature node i has, or that it has none ( =0)

INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read from the file.
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write to the file.

np1 = n+1

# in the next block, we choose which nodes will have features, and what the features are.
for i in range(1, np1):  # start of block 1
     print "%d " % i
     randomnum = random.randrange(101)
     print ("%d, %d") % (i,randomnum)
     if (randomnum < 50):         # about half of the nodes will have a feature.
          has_feature.append(1)
          frand = random.randrange(1,5)   # randomly choose a feature from the choices 1,2,3,4.
          print ("A random feature was chosen:  %d") % frand
          feature.append(frand)
          constraints = constraints + "X(%d,%d) = 1 \n" % (i, frand) # X(i, frand) records whether or not node i has
                                                                     # feature frand.
          binaries = binaries + "X(%d,%d) \n" % (i, frand)

     else:
         has_feature.append(0)
         feature.append(0)
         for f in (1,2,3,4):
             constraints = constraints + "+ X(%d,%d) " % (i,f)  # set up equality to ensure that node i gets
                                                                # assigned a feature
         constraints = constraints + "= 1 \n"
         binaries = binaries + "X(%d,%d) \n" % (i, f)

i = 0
for line in INFILE:
     i = i + 1
     j = 0
     for char in line:
         j = j + 1
         if (char == '1'):  # (i,j) is an edge

              if ((feature[i-1] == 0) and (feature[j-1] == 0)): 
                  for f in (1,2,3,4):
                      constraints = constraints + "B(%d,%d,%d) - X(%d,%d) <= 0 \n" % (i,j,f,i,f)
                      constraints = constraints + "B(%d,%d,%d) - X(%d,%d) <= 0 \n" % (i,j,f,j,f)
#                    constraints = constraints + "X(%d,%d) + X(%d,%d) - B(%d,%d,%d) <= 1 \n" % (i,f,j,f,i,j,f)
                      objective = objective + "+ B(%d,%d,%d) " % (i,j,f)
                      binaries = binaries + "B(%d,%d,%d) \n" % (i,j,f)
              if ((feature[i-1] == 0) and (feature[j-1] != 0)): 
                  g = feature[j-1]
                  constraints = constraints + "B(%d,%d,%d) - X(%d,%d) <= 0 \n" % (i,j,g,i,g)
                  objective = objective + "+ B(%d,%d,%d) " % (i,j,g)

              if ((feature[j-1] == 0) and (feature[i-1] != 0)): 
                  g = feature[i-1]
                  constraints = constraints + "B(%d,%d,%d) - X(%d,%d) <= 0 \n" % (i,j,g,j,g)
                  objective = objective + "+ B(%d,%d,%d) " % (i,j,g)

              if ((feature[j-1] != 0) and (feature[i-1] != 0)): 
                   if (feature[i-1] == feature[j-1]):
                      g = feature[i-1]
                      constraints = constraints + "B(%d,%d,%d) = 1 \n" % (i,j,g)
                      objective = objective + "+ B(%d,%d,%d) " % (i,j,g)

#end of block 1
print (has_feature)
print (feature)
OUTFILE.write ("Maximize \n")
OUTFILE.write (objective + "\n")
OUTFILE.write ("such that \n\n")
OUTFILE.write(constraints)
OUTFILE.write("\nbinary \n")
OUTFILE.write(binaries)
