# Aparsimony.pl
#
# March 3, 2017 Dan Gusfield
#
# Perl program to create the concrete ILP formulation for the maximum parsimony problem,
# given a set of binary sequences.
#
# Call on a command line in a text-window as:
# perl Aparsimony.pl sequence-file-name length-of-sequences
#
# Example Data: bookpars
#
#
$constraintcount = 0;
$variablecount = 0;

my $bits = $ARGV[1];
@charsize = ();

  for $i (1 .. $bits) { 
      $charsize[$i] = 0;
  }

@lines = ();

open (IN, $ARGV[0]);
open (OUT, ">$ARGV[0].lp");
$i = 0;

while ($line = <IN>) {

  push(@lines, $line);
#  $lines[$i] = $line;
#  print "$line";
  $i++;
  chomp $line;
  $sequence{$line} = 1; # put the input lines into the hash %sequence

  } # end while line = <IN>


$numseqs = @lines;
print "$numseqs, $bits \n";  # the number of rows and columns in the input matrix

$binaries = "";
$generalF = "";
$constraints = "";
%bin2dec = {};

my @binary_list = ();  # generate all of the 2^bits binary strings from 0 to 2^bits - 1.
my $remainder;

my $bound = 2**$bits - 1;
for my $num (0 .. $bound) {
my $dec_number = $num;

my $result = "";

my $iters = 0;
while ($dec_number >= 1)
{
    $remainder = $dec_number % 2;       #Modulo division to get remainder
    $result = $remainder . $result;     #Concatenation of two numbers
    $dec_number = $dec_number / 2;     #New Value of decimal number to do next set of above operations
    $iters++;
}

while ($iters < $bits) {               # fill out the leftmost bits with 0s as needed to get the right
                                       # number of bits
    $result = '0' . $result;
    $iters++;
}

#print "Binary number Output = ", $result, "\n";
push(@binary_list, $result);
$bin2dec{$result} = $num; # populate has bin2dec with binary index and decimal value
}

#print "@binary_list \n";



%edgehash = ();
   #create the neighbor list for $num, i.e., the list of nodes that have one fewer bit than $num, and are neighbors
   # in the hypercube. So, those edges are into $num.

 foreach $num (@binary_list) {
#     print "$bin2dec{$num}   $num \n";

     $into = "";
     $outof = "";
     $Xinto = "";

     @bnum = split (//, $num);

     foreach $i (0 .. $bits - 1) {
         if ($bnum[$i]  == 1) {    # generate the flow-in inequalities because bit i is 1
            @altnum = @bnum;
            $altnum[$i] = 0;   # generate altnum which differs from num by changing 1 to 0 at bit i
            $baltnum = join ('',@altnum);
#            print ("Into $num from $baltnum \n");
            $Xkey = 'A' . $bin2dec{$baltnum} . ',' . 'A' . $bin2dec{$num};  # create the key needed for edgehash and $into and $Xinto.  change 4/23/17
            $into .= "+ F($Xkey) "; # flow for the steiner tee, from baltnum into num
            $Xinto .= "+ X($Xkey) "; # list of X variables for edges into num.
            $edgehash{$Xkey} = 1;

         }   
         else {                  # generate the flow-out inequalities because bnum[i] == 0, so its out neighbors differ
                                 # by having a 1 in position i.
            @altnum = @bnum;
            $altnum[$i] = 1;
            $baltnum = join ('',@altnum);
#            print ("Out $num, $baltnum \n");
            $outof .= "- F(A$bin2dec{$num},A$bin2dec{$baltnum}) ";
         }
      }  # end of the interior bits loop

  if ($num eq $binary_list[0]) { # case for outflow from the root of the hypercube
#       print OUT "$outof <= -$numseqs \n";
       $constraints .= "$outof <= -$numseqs \n\n"; # I don't see now why this is needed.
       }

  elsif ($sequence{$num} == 1) {  # if $num equals an input row, require that it be reached in the Steiner tree, which means that
                               # the flow into it is one more than the flow out of it.
                               #
#     print OUT "$into" . "$outof = 1 \n";
      $constraints .= "$into" . "$outof = 1 \n";
      $constraintcount++;
  }

  else {
#           print OUT "$into" . "$outof = 0 \n";  # Kirkoff balance equation for interior nodes
           $constraints .=  "$into" . "$outof = 0 \n";  # Kirkoff balance equation for interior nodes
           $constraintcount++;
       }

#   print OUT "$Xstring <= 1\n";   # inequality that says there can be at most one edge into $num in the steiner tree.
   if ($Xinto) {
       $constraintcount++;
       $constraints .=  "$Xinto <= 1\n";   # inequality that says there can be at most one edge into $num in the steiner tree.
   }

 } # end foreach num in binary_list

#print OUT "hybercube edges\n";
print OUT "Minimize \n"; 

foreach $Xkey (sort keys %edgehash) {
     $variablecount++;
     print OUT "+ X($Xkey) ";
}
print OUT "\n\n subject to \n";
print OUT $constraints;

foreach $Xkey (sort keys %edgehash) {
#  print OUT "$key \n";
#            print OUT ("X($Xkey) - F($Xkey) <= 0 \n");  # set X($Xkey) to zero if there is no flow in edge $Xkey.
            $constraintcount++;
            print OUT ("F($Xkey) - $numseqs X($Xkey) <= 0 \n"); # set X($key) to one if there is flow in edge $Xkey.
            $binaries .= "X($Xkey)\n";
            $variablecount++;
            $generalF .= "F($Xkey)\n";
}

print OUT "general\n";
print OUT "$generalF";
print OUT "binary\n";
print OUT "$binaries";
print OUT "end\n";

print "variables, constraints: $variablecount, $constraintcount\n";
print "The ILP file is $ARGV[0].lp \n";
