# BBparsimony.pl
# April 28, now just use nodes and edges in the Buneman graph.
#
# DG this program assumes that the all-zero sequence will be the root. 
# April 27, 2019 Previously, if the all-zero sequence was also an input sequence,
# the ILP produced would be infeasible. This has been fixed. In fact, now if you
# want to restrict the underlying graph to the Buneman graph formed from the input
# sequences, you have to include the all-zero sequence in the input. If we extend
# the program so that the root is not specified, then that assumption can be dropped.
# 
#
# March 3, 2017 Dan Gusfield
#
# Perl program to create the concrete ILP formulation for the maximum parsimony problem,
# given a set of binary sequences. It first removes duplicate cols. Also, while
# constructing the hypercube on which the maximum parsimony tree will be found,
# it determines which of the nodes in the hypercube are Buneman nodes.
#
# Call on a command line in a text-window as:
# perl BBparsimony.pl sequence-file-name length-of-sequences
#
# Example Data: bookpars
#
#
$constraintcount = 0;
$variablecount = 0;

my $numbits = $ARGV[1];

@inlines = ();
@lines = ();
%Bnodes = (); # hash to accumulate the Buneman nodes

open (IN, $ARGV[0]);
open (OUT, ">$ARGV[0].lp");
open (TRACE, ">t$ARGV[0]");
$numseqs = 0;
$allzero = 0;
while ($line = <IN>) {

  if (not $line =~ /1/) {     # examine each line to see if it is the all-zero sequence
      print "All zeros $line";
      $allzero = 1;
  }

#  print TRACE "$line";
  chomp $line;
  push(@lines, $line);
  $numseqs++;
  $sequence{$line} = 1; # put the input lines into the hash %sequence
  $Bnodes{$line} = 1; # put the input lines into a hash of all the Buneman nodes

  } # end while line = <IN>

$root = $lines[0]; # The first sequence in the input is taken as the root node.
%pairs = ();  # a hash holding the column pairs together with binary pairs in the input. This is used
              # to test if a binary sequence defines a Buneman node.
$Bnum = 0; # number of Buneman nodes found so far



%NotBnodes = (); # visited binary sequences that are not Buneman nodes

@expandlist = (); # list of Buneman nodes that need to be expanded to find neighbors. This is organized
                  # as a queue, i.e., first-in first-out.
%expanded = (); # a hash of binary Buneman sequences that have been taken out of expandlist and used to find outher Buneman
                # nodes.
$num_nodes = 0; # a count of the current expandlist 

foreach $node  (sort keys %Bnodes) {

     @bits = split(//, $node);
     $numbits = @bits;
     print TRACE "This is where numbits is set, and it has value $numbits \n";

     push (@expandlist, $node);
     $num_nodes = $num_nodes + 1;

     foreach $i (0 ... $numbits -2) {
         foreach $j ($i+1...$numbits - 1) {
            $ii = $i+1;
            $jj = $j+1;
            $key = $ii . ',' . $jj . ',' . $bits[$i] . ',' . $bits[$j];  # a key has two column indices and two
                                                                         # bits that are in those columns for some
                                                                         # input row
            $pairs{$key} = 1;
         }
      }
}


print TRACE "The number of bits is $numbits \n";
print TRACE "\n";

print TRACE "The initial Buneman nodes from input are \n";
foreach $node (sort keys %Bnodes) {  # print the Buneman nodes that are found
      print TRACE "$node \n";
      $Bnum = $Bnum + 1;
}
print "\n";


print TRACE "The number of input Buneman nodes is $Bnum \n";
print TRACE "\n";


# Now we do the expansion search from each known Buneman nodes to find all Buneman nodes
$binaries = "";
$generalF = "";
$constraints = "";
%bin2dec = {};

$edgecount = 0;
while ($num_nodes > 0) {
   $node =  shift (@expandlist); # pop from the head of the queue expandlist
   $expanded{$node} = 1;  # when a node is popped off the queue it is declared expanded, even though the
                          # full expansion of the node has not happened yet.
   print TRACE "popped out of expandlist: $node $bin2dec{$node} \n";
   $num_nodes = $num_nodes - 1;

   print TRACE "\n";
   @bits = split(//, $node);
   @savbits = @bits;

BT:foreach $i (0 ... $numbits - 1) {
      @bits = @savbits;
      $bits[$i] = 1 - $bits[$i];  # reverse the bit at position i
      $newsequence = join ('',@bits); # this creates a binary sequence that differs from 
                                  # node in exactly one bit. It might be a Buneman node, or not.
#      print TRACE "\n The Bnode and a tested sequence \n $node \n $newsequence \n\n";

      if ((defined $Bnodes{$newsequence}) and (!defined $expanded{$newsequence}))   {
          $edgecount ++;
          @splitbits = split (//,$node);
          $A = bin2decnum(@splitbits);
          @splitbits = split (//,$newsequence);
          $B = bin2decnum(@splitbits);
          print TRACE "$newsequence, $B is an already  known Buneman node \n";
          print TRACE "$A, $B is a Buneman edge, number $edgecount \n";
      }
      else {
        if  (defined $NotBnodes{$newsequence}) {
            print TRACE "$newsequence is already known not to be a Buneman node \n";
        }

      else {

        if ((!defined $Bnodes{$newsequence}) and (!defined $NotBnodes{$newsequence})) {  
                                             # if the newsequence has not been identified as a Buneman node or a non-Buneman
                                             # node, test it.
                                             # This will keep from discovering the
                                             # same Buneman node more than once. And also avoids testing any non-Buneman
                                             # node more than once.

            foreach $p (0 ... $numbits -2) {  # test if newsequence is a Buneman node
                 foreach $q ($p+1...$numbits - 1) {
                     $ii = $p+1;
                     $jj = $q+1;
                     $key = $ii . ',' . $jj . ',' . $bits[$p] . ',' . $bits[$q];  
                                                                         # a key has two column indices and two
                                                                         # bits that are in those columns for some
                                                                         # input row
                     if (!defined $pairs{$key}) {   # end the test if any required pair is missing from the input
                                                    # sequences
                         print TRACE "$newsequence $order{$newsequence} is not a Buneman node. $key is missing\n";
                         $NotBnodes{$newsequence} = 1;
                         next BT;
                     }
                 }
            }
        push (@expandlist, $newsequence);  # the newsequence passed the tests for a Buneman node, so push it onto the
                                           # tail 
                                           # the queue expandlist, and increment the count of expandlist
        $num_nodes = $num_nodes + 1;
        $Bnodes{$newsequence} = 1;     # insert the newsequence into the hash of Buneman nodes
        $Bnum = $Bnum + 1;
#        print "YAY! $newsequence is a Buneman node, count $Bnum \n";
        print TRACE "YAY! $newsequence is a Buneman node, count $Bnum \n";
        $edgecount ++;
          @splitbits = split (//,$node);
          $A = bin2decnum(@splitbits);
          @splitbits = split (//,$newsequence);
          $B = bin2decnum(@splitbits);
#          print TRACE "$A, $B is a Buneman edge, number $edgecount \n";
      }
     }
  }
 } # end of foreach $i
} # end of  while num_nodes
            
print "The number of nodes is $Bnum, and the number of edges is $edgecount \n";
print TRACE "The number of Buneman nodes is $Bnum \n";

%order = ();
$Bunorder = 1;
print TRACE "ZZ The Buneman nodes are: \n";
foreach $node (sort keys %Bnodes) {
        @nodebits = split (//,$node);
        $decnum = bin2decnum(@nodebits); # compute the decimal representation of a bnode.
        $bin2dec{$node} = $decnum;
        $order{$node} = $Bunorder; # give a sequential ordering to the nodes both to their binary number
                                   # the order numbers are not the same as the Bnum numbers used earlier
        $order{$decnum} = $Bunorder; # representation in $node, and their decimal representation in $decnum
        print TRACE "Node representations: $node $decnum $Bunorder\n";
        $Bunorder = $Bunorder + 1;
}
$Bunorder = $Bunorder - 1;
print TRACE "WW The last trans number is $Bunorder \n";

%edgehash = ();

   #create the neighbor list for $num.  The nodes that have one fewer bit than $num, and are neighbors of $num
   # are ends of edges into $num. The nodes with one more bit, and are neighbors, are ends of edges out of $num.

  foreach $num (sort keys %Bnodes) {  # scan through the Buneman nodes in binary
      print TRACE "Scan for neighbors of $num $bin2dec{$num}   num $order{$num} \n";

     $into = "";
     $outof = "";
     $Xinto = "";

     @bnum = split (//, $num); # split the string into its bits

     foreach $i (0 .. $numbits - 1) {
         if ($bnum[$i]  == 1) {    # generate the flow-in inequalities because bit i is 1
            @altnum = @bnum;
            $altnum[$i] = 0;   # generate altnum which differs from num by changing 1 to 0 at bit i
            $baltnum = join ('',@altnum); # create the binary string from $altnum
            if (defined $Bnodes{$baltnum}) { # check if $baltnum is a Buneman node
              print TRACE ("Decrep: Into $bin2dec{$num} from $bin2dec{$baltnum} \n");
              print TRACE ("Orderrep: Into $order{$num} from $order{$baltnum} \n");
                $Xkey = 'A' . $order{$baltnum} . ',' . 'A' . $order{$num};  # create the key needed for edgehash and $into and $Xinto.  
#                $Xkey = 'A' . $bin2dec{$baltnum} . ',' . 'A' . $bin2dec{$num};  # create the key needed for edgehash and $into and $Xinto.  
                $into .= "+ F($Xkey) "; # flow for the steiner tree, from baltnum into num
                $Xinto .= "+ X($Xkey) "; # list of X variables for edges into num.
                $edgehash{$Xkey} = 1;
             }
         }   
         else {                  # generate the flow-out inequalities because bnum[i] == 0, so its out neighbors differ
                                 # by having a 1 in position i.
            @altnum = @bnum;
            $altnum[$i] = 1;
            $baltnum = join ('',@altnum);
            $A = bin2decnum(@bnum);
            $B = bin2decnum(@altnum);


            if (defined $Bnodes{$baltnum}) { # check if $baltnum is a Buneman node
#                print TRACE ("Out from $order{$A} to $order{$B} \n");
                print TRACE ("Out from $order{$num} to $order{$baltnum} \n");
                print TRACE "defined  $baltnum, $bin2dec{$baltnum}, $order{$baltnum}\n";
                $Xkey = 'A' . $order{$num} . ',' . 'A' . $order{$baltnum};  # create the key needed for edgehash and $outof.  
#                $Xkey = 'A' . $bin2dec{$num} . ',' . 'A' . $bin2dec{$baltnum};  # create the key needed for edgehash and $outof.  
                $outof .= "- F($Xkey) "; # flow for the steiner tree, from num to baltnum
                $Xoutof .= "+ X($Xkey) "; # list of X variables for edges out of $num. # May 10, 2019
                $edgehash{$Xkey} = 1;

            } 
         }
#      print TRACE "After numbits loop  $outof \n";
      }  # end of the interior numbits loop

  if ($num eq $root) { # case for outflow from the root (all-zero sequence) of the hypercube
       $rhs = $numseqs - $allzero; # subtract 1 if the all-zero sequence is an input sequence
       $constraints .= "$outof <= -$rhs\n\n"; 
       $constraintcount++;
       }

  elsif ($sequence{$num} == 1) {  # if $num equals an input row, require that it be reached in the Steiner tree, which means that
                               # the flow into it is one more than the flow out of it.
                               #
      if ($into eq "")  {
          print TRACE "YIKES $num $order{$num} $bin2dec{$num}\n";
      }

      print TRACE "Again INTO: $into OUTOF: $outof \n";
      $constraints .= "$into" . "$outof  = 1 \n";
      $constraintcount++;
  }

  else {
           $constraints .=  "$into" . "$outof = 0 \n";  # Kirkoff balance equation for interior nodes
           $constraintcount++;
       }

   if ($Xinto) {
       $constraintcount++;
       $constraints .=  "$Xinto <= 1\n";   # inequality that says there can be at most one edge into $num in the steiner tree.
   }

 } # end foreach num in binary_list, or for each node in keys of Bnodes

print OUT "Minimize \n"; 

foreach $Xkey (sort keys %edgehash) {
     $variablecount++;
     print OUT "+ X($Xkey) ";
}
print OUT "\n\n subject to \n";
print OUT $constraints;

foreach $Xkey (sort keys %edgehash) {
            $constraintcount++;
            print OUT ("F($Xkey) - $numseqs X($Xkey) <= 0 \n"); # set X($key) to one if there is flow in edge $Xkey.
            $binaries .= "X($Xkey)\n";
            $variablecount++;
            $generalF .= "F($Xkey)\n";
}

print OUT "general\n";
print OUT "$generalF";
print OUT "binary\n";
print OUT "$binaries";
print OUT "end\n";

print "variables, constraints: $variablecount, $constraintcount\n";
print "The ILP file is $ARGV[0].lp \n";

#######################

sub cleandata {
# remove duplicate rows and cols.

@inlines = @_;

%insequence = ();

foreach $line (@inlines) {  # remove duplicate rows in the original input sequences
   chomp $line;
   print "$line \n";
   if (!defined $insequence{$line}) {
      $insequence{$line} = 1;
      push (@lines, $line);
   }
}


%sequences = ();
@cleanlines = ();
@tlines = transpose(@lines);  # transpose the input matrix @lines

foreach $line (@tlines) {   # remove duplicate rows in the transposed matrix, which
                                 # removes duplicate cols in the original matrix
#   print "$line \n";
   if (!defined $sequences{$line}) {
      $sequences{$line} = 1;
      push (@cleanlines, $line);
   }
}

@flines = transpose(@cleanlines);  # now transpose back without the duplicates
foreach $line (@flines) {
   print CRES "$line\n";
}

return @flines;

}


#####################
sub transpose {

@lines = @_;

@transpose = ();
$i = 0;
foreach $line (@lines) {
  chomp $line;
  @row = split(//,$line);
  foreach $j (0 ... length($line) - 1) {
     $transpose[$j][$i] = $row[$j];
#     print "$i, $j, $row[$j], $transpose[$j][$i]\n";
  }
# print "\n";
$i = $i + 1;
$linelength = length($line);
}

 print "The number of rows in the transpose is $linelength, and the number of cols. is $i \n";

 @trans = ();
 foreach $p (0 ... $linelength - 1) {
    $newrow = "";
    foreach $q (0 ... $i-1) {
#      print "$transpose[$p][$q]";
      $newrow = $newrow . $transpose[$p][$q];
    }
#    print "\n";
    push (@trans, $newrow);
 }

# print "XXXXXX\n";
# foreach $line (sort @trans) {
#      print CRES "$line\n";
# }

return @trans;
}

####################
 

sub bin2decnum {
# convert a binary string to decimal
# The input must be a list of the bits in the
# string, not  a single string, which is a scalar

@bits = @_;
#print "@bits \n";

$numbits = @bits;
$last = $numbits - 1;
$power = 1;
$dec = 0;

foreach $i (0 ... $last) {
    $index = $last - $i;
    $dec = $dec + $power * $bits[$index];
    $power = $power * 2;
}

# print "exiting bin2decnum $dec \n";
return int($dec);

}

