# CL2graphfile.py written for Python 2.7
#
# DG Nov 3, 2017
#
# This Python program reads a file containing the adjacency matrix of a graph 
# and creates the concrete ILP for the problem of finding
# two cliques. One is a largest clique, K, and the other is a largest clique
# that shares *no* nodes with K.
#

# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

arg1 = sys.argv[1]    # assign the first argument (naming the input file) to the variable arg1
arg2 = sys.argv[2]    # assign the second argument (naming the ILP output file) to the variable arg2
                      # The values in arg1 and arg2 are strings.

INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read in the adjacency matrix 
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write the concrete ILP formulation to the file.

constraints = "such that \n\n"  # assign the string `such that \n\n' to the variable `constraints'
listC = ""                      # assign the empty string to the variable `listC'
listD = ""                      # assign the empty string to the variable `listC'
binaries = "binary \n"          # assign the string `binary \n' to the variable `binaries'

i = 0
for line in INFILE:  # read in the lines from INFILE, one line at a time
    i = i + 1   # increment i to keep track of the line number just read in.
    listC = listC + "+ C(%d) " % i   # concatenate the string "+ C(i) " to the variable 'listC'
    listD = listD + "+ D(%d) " % i   # concatenate the string "+ D(i) " to the variable 'listD'
    binaries = binaries + "C(%d)\n" % i # and concatenate "+ C(i) \n" to the variable 'binaries'
    binaries = binaries + "D(%d)\n" % i # and concatenate "+ D(i) \n" to the variable 'binaries'

# comment out the next line if you want to allow K and K' to share some nodes. This is fine if K and K'
# are the same size, but if K' is smaller than K, the solution might just find a K' that is a subset of K.

    constraints = constraints + "C(%d) + D(%d) <= 1 \n" % (i,i) # create the constraints that force K and K' to
                                                                # have *no* nodes in common

    j = 0    
    for char in line:   # for each character in the line just read in:
       j = j + 1    # increment j to keep track of the character position being examined.
       if char == '0' and (i < j):  # if the char is a '0', create the next string. But we don't
                                    # want to generate the same inequality twice, so we only
                                    # do it if i < j.
          string = "C(%d) + C(%d) <= 1\n" % (i,j)  # create the needed inequality for i and j
          constraints = constraints + string   # concatenate that inequality to the variable 'constraints'

          string = "D(%d) + D(%d) <= 1\n" % (i,j)  # create the needed inequality for i and j
          constraints = constraints + string   # concatenate that inequality to the variable 'constraints'

Cstring = listC + "- C = 0 \n" # set variable C to the sum of the C(i) variables. 
Dstring = listD + "- D = 0 \n" # set variable D to the sum of the D(i) variables.
CDdiff = "C - D >= 0 \n"        # the sum of the C variables must be larger or equal to the sum of the D variables
                                # This forces clique K to be specified by the C variables, and clique K' to be specified
                                # by the D variables, if the size of K' is smaller than the size of K.
INFILE.close()
OUTFILE.write("Maximize \n") # write to file the string (word) 'Maximize' and move to a new line (because of '\n')
OUTFILE.write(listC + "\n")  # write to file the value of variable 'listC' and move to a new line
OUTFILE.write(listD + "\n")  # write to file the value of variable 'listC' and move to a new line

OUTFILE.write (constraints)  # write to file the value of variable 'constraints'
OUTFILE.write(Cstring) # write out the inequality that sets ILP variable C to the sum of the C(i) variables.
OUTFILE.write(Dstring) # write out the inequality that sets ILP variable D to the sum of the D(i) variables.
OUTFILE.write(CDdiff)  # write out the inequality specified by the string CDdiff
OUTFILE.write (binaries)     # write to file the value of the variable 'binaries'
OUTFILE.write ("end")        # write to file the string (word) 'end'
OUTFILE.close()
