# CLgraphfile.py written for Python 2.7
#
# DG October 9, 2017

# This Python program reads a file containing the adjacency-matrix of a graph,
# and creates the concrete ILP for the maximum clique problem  for that graph.
# The program assumes that the adjacency matrix of the graph has no spaces between entries.

# Call on a command line in a terminal window as:

# python CLgraphfile.py adjacency-matrix-file ILP-file.lp
# Example data: example-graph


# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

arg1 = sys.argv[1]    # assign the first argument  to the variable arg1
arg2 = sys.argv[2]    # assign the second argument to the variable arg2
                      # The values in arg1 and arg2 are strings.

INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read the adjacency matrix of a graph from the file.
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write the concrete ILP formulation to the file.

constraints = "such that \n\n"  # assign the string `such that \n\n' to the variable `constraints'
listC = ""                      # assign the empty string to the variable `listC'
binaries = "binary \n"          # assign the string `binary \n' to the variable `binaries'

index = 0
for line in INFILE:  # read in the lines from INFILE, one line at a time
    index = index + 1   # increment i to keep track of the line number just read in.
    listC = listC + "+ C(%d) " % index   # concatenate the string "+ C(i) " to the variable 'listC'
    binaries = binaries + "C(%d)\n" % index # and concatenate "+ C(i) \n" to the variable 'binaries'

    j = 0    
    for char in line:   # for each character in the line just read in:
       j = j + 1    # increment j to keep track of the character position being examined.
       if char == '0' and (index < j):  # if the char is a '0', create the next string. But we don't
                                    # want to generate the same inequality twice, so we only
                                    # do it if index < j.
          string = "C(%d) + C(%d) <= 1\n" % (index,j)  # create the needed inequality for index and j
          constraints = constraints + string   # concatenate that inequality to the variable 'constraints'

INFILE.close()
OUTFILE.write("Maximize \n") # write to file the string (word) 'Maximize' and move to a new line (because of '\n')
OUTFILE.write(listC + "\n")  # write to file the value of variable 'listC' and move to a new line
OUTFILE.write (constraints)  # write to file the value of variable 'constraints'
OUTFILE.write (binaries)     # write to file the value of the variable 'binaries'
OUTFILE.write ("end")        # write to file the string (word) 'end'
OUTFILE.close()

print ("The ILP file is: %s \n" % arg2);
