# LCS_Mult.py
#
# Written by Jessica Au and Jessie Vuon as part of their class project in the class Integer Linear Programming in
# Computational Biology, Spring quarter 2017.
#
# This is for Python 2.7
#
# This python script is meant to generate the ILP formulation to solve the LCS problem for MULTIPLE strings.
# Currently, it only works for THREE strings.
#
# Call this program on command line as: python2.7 LCS_Mult.py DNA.txt LP_outputfile.lp minDist num-strings
# Inputs:
#	DNA_file.txt, a plain text file containing three DNA sequences, each with a max of 100 nucleotides.
#	minDist, the minimum distance between matches, we can consider this as spaces/indels. Set this to 0 for
#       the basic LCS problem.
#       num-strings is the number of strings input. Currently, use 3.
# Output:
#	LP_outputfile.lp

# Example input files: test3seqs.txt, multiple_string.txt
#
# import sys module

import sys

arg1 = sys.argv[1]
arg2 = sys.argv[2]
minD = sys.argv[3]
numS = sys.argv[4]

numS = int(numS)
minD = int(minD) # this will be set to 0 for a minimum of no spaces. For basic LCS, set minD to 0.
listP = "" # Variable will be set to 1 when there is a match.
listU = ""
listS = ""


INFILE = open(arg1, "r")
OUTFILE = open(arg2, "w")

# Assume DNA sequences are of EQUAL length
n = 0

# be sure to read separate sequences.
Strings = []

for line in INFILE:

    if (n==0):
        Strings.append(line.rstrip())
        n += 1
    elif (n!=0):
        Strings.append(line.rstrip()[::-1])
    print "You input the sequence: ", line.rstrip()
    SeqLength = len(line.rstrip())

# Put together the sequences
DNASequence = []
placeholder = []
for s in range(1, numS):
    placeholder.append(Strings[0])
    placeholder.append(Strings[s])
    placeholder = "".join(placeholder)
    DNASequence.append(placeholder)
    placeholder = []

# print "Your sequences are: ", DNASequence

DNAlength = len(DNASequence[0]) # They should be equal length
local_range = SeqLength + 1
global_range = DNAlength + 1
#print (local_range)
#print (global_range)
legal = [[[0 for x in range(1, global_range + 1)]for y in range(1, global_range + 1)] for z in range(1, global_range+1)]

# Array of [[22]22]
#print (legal) # make sure this is correct
for k in range(0, numS-1):
    for i in range(1, local_range):
        for j in range(local_range, global_range):
            #print(j-i, minD)
            if (j-i > minD):
                if (DNASequence[k][i-1] == 'A') and (DNASequence[k][j-1] == 'A'):
                    #print (DNASequence[k][i - 1], i - 1, j - 1, k)
                    legal[i][j][k] = 1
                    #OUTFILE.write("V(%d,%d) = 0 \n" % (i,j))
                if (DNASequence[k][i-1] == 'T') and (DNASequence[k][j-1] == 'T'):
                    #print (DNASequence[k][i - 1], i - 1, j - 1, k)
                    legal[i][j][k] = 1
                    #OUTFILE.write("V(%d,%d) = 0 \n" % (i,j))
                if (DNASequence[k][i-1] == 'C') and (DNASequence[k][j-1] == 'C'):
                    #print (DNASequence[k][i - 1], i - 1, j - 1, k)
                    legal[i][j][k] = 1
                    #OUTFILE.write("V(%d,%d) = 0 \n" % (i,j))
                if (DNASequence[k][i-1] == 'G') and (DNASequence[k][j-1] == 'G'):
                    #print (DNASequence[k][i - 1], i - 1, j - 1, k)
                    legal[i][j][k] = 1
                    #OUTFILE.write("V(%d,%d) = 0 \n" % (i,j))
                legal[i][j][k] = legal[i][j][k]



# print(legal)
# Print out the Objective Function
OUTFILE.write("Maximize \n")

# Accumulate a list of P variables for our legal pairs.
for i in range(1, local_range):
    listS = listS + "+S(%d,%d) " %(i,0)
    for k in range(0, numS-1):

        listU = listU + "+U(%d,%d) " % (i,k)
        for j in range(local_range, global_range):
            if (legal[i][j][k] == 1):
                listP = listP + "+P(%d,%d,%d) " % (i,j,k)


                #print (i, j, k, legal[i][j][k])

#OUTFILE.write ("\nP + U + S\n")
#OUTFILE.write ("\nP \n")
#OUTFILE.write ("\nU \n")
OUTFILE.write ("\nS \n")
OUTFILE.write ("\nsuch that \n")

# produce an equality that sets ILP variable P to the sum of the P(i,j,k) variables.
OUTFILE.write (listU + "- U = 0 \n\n")
OUTFILE.write (listP + "- P = 0 \n\n")
OUTFILE.write (listS + "- S = 0 \n\n")


# This to ensure that only one nucleotide is matched
# Check this again.
for k in range(0, numS-1):

    for i in range(1,global_range):

        inequality = ""

        for j in range(1, local_range):
            if (legal[j][i][k]):
                inequality = inequality + " + P(%d,%d,%d)" % (j,i,k)

        for j in range(local_range,global_range):
            if (legal[i][j][k]):
                inequality = inequality + " + P(%d,%d,%d)" % (i,j,k)

        inequality = inequality + ' <= 1'
        OUTFILE.write (inequality)
        OUTFILE.write ("\n\n")

#  Now check for crossings between strings
for l in range (0, numS-1):
    for h in range(1, local_range-1):
        for i in range(h+1, local_range):
            for j in range(local_range, global_range - 1):
                for k in range(j+1, global_range):
                   if ((legal[h][j][l] == 1) and (legal[i][k][l])):
                        OUTFILE.write ("P(%d,%d,%d) + P(%d,%d,%d) <= 1 \n" % (h,j,l,i,k,l))


# Extra constraint to check whether they are equal in both strings.
# This to is to indicate that P(i,j,k) is 1 for some j in s_k


for k in range(0, numS-1):
    for i in range(1, local_range):

        inequality2 = "U(%d,%d)" %(i,k)

        for j in range(local_range, global_range):
            if (legal[i][j][k]):
                inequality2 = inequality2 + " - P(%d,%d,%d)" % (i,j,k)


        inequality2 = inequality2 + ' <= 0'
        OUTFILE.write (inequality2)
        OUTFILE.write ("\n\n")

# Another constraint to make sure that this appears in all strings.
for k in range(0,numS-2):
    for i in range(1,local_range):
        OUTFILE.write("2 S(%d,%d) - U(%d,%d) - U(%d,%d) <= 0 \n" %(i,k,i,k,i,k+1) )

OUTFILE.write("\n")
OUTFILE.write ("binary \n")
for i in range(1, local_range):
    OUTFILE.write("S(%d,%d) \n" %(i,0))
    for k in range(0, numS-1):
        OUTFILE.write("U(%d,%d) \n" % (i,k))
        for j in range(local_range, global_range):
            if (legal[i][j][k]):
                OUTFILE.write("P(%d,%d,%d) \n" % (i,j,k))


OUTFILE.write("\n")
OUTFILE.write ("general\nS\n")
OUTFILE.write ("general\nU\n")
OUTFILE.write  ("end \n")
print "The ILP is in file %s" % arg2
INFILE.close()
