# LDNclique.py written for Python 2.7
#
# This Python program reads a file with the adjacency matrix of a graph, 
# and creates the concrete ILP for the Largest Dense-Near-clique problem,
# for that graph.
# DG October 23, 2017

# Call on a command line in a terminal window as:

# python LDNclique.py graph-file ILP-file.lp fraction
# where ``fraction" is the fraction of new edges compared to original edges in the created clique.

# Example data: example-graph 

# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

arg1 = sys.argv[1]    # assign the first argument to the variable arg1
arg2 = sys.argv[2]    # assign the second argument to the variable arg2
                      # The values in arg1 and arg2 are strings.
r = sys.argv[3]       # r is the multiplicitive factor for the Y variables
p = float(r)

INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read the adjacency-matrix of the graph from the file.
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write to the  ILP.lp file.

Vconstraints = "such that \n\n"  # assign the string `such that \n\n' to the variable `Vconstraints'
Yconstraints = ""               # assign the empty string to the variable `Yconstraints'
listC = ""                      # assign the empty string to the variable `listC'
listV = ""                      # assign the empty string to the variable `listV' 
listY = ""                      # assign the empty string to the variable `listY' 
Cbinaries = ""                  # assign the empty string to the variable `Cbinaries'
Vbinaries = ""                  # assign the empty string to the variable `Vbinaries'
Ybinaries = ""                  # assign the empty string to the variable `Ybinaries'

i = 0
for line in INFILE:  # read in the lines from INFILE, one line at a time
    i = i + 1   # increment i to keep track of the line number just read in.
    listC = listC + "+ C(%d) " % i   # concatenate the string "+ C(i) " to the variable 'listC'
    Cbinaries = Cbinaries + "C(%d)\n" % i # and concatenate "+ C(i) \n" to the variable 'Cbinaries'

    j = 0    
    for char in line:   # for each character in the line just read in:
       j = j + 1    # increment j to keep track of the character position being examined.
       if char == '0' and (i < j):  # if the char is a '0', create the next string. But we don't
                                    # want to generate the same inequality twice, so we only
                                    # do it if i < j.

          Vstring = "C(%d) + C(%d) - V(%d,%d) <= 1\n" % (i,j,i,j)  # create the needed inequality for i and j
                                                                  # so if C(i) and C(j) are both set to 1,
                                                                  # variable V(i,j) will also be set to 1.

          Vconstraints = Vconstraints + Vstring   # concatenate that inequality to the variable 'Vconstraints'

          listV = listV + "+ V(%d,%d) " % (i,j)   # concatenate the string "+ V(i,j) " to the variable 'listV'
          Vbinaries = Vbinaries + "V(%d,%d) \n" % (i,j) # and concatenate "+ V(i,j) \n" to the variable 'Vbinaries'

       if char == '1' and (i < j):  # if the char is a '1', create the next string. But we don't
                                    # want to generate the same inequality twice, so we only
                                    # do it if i < j.

          Ystring = "C(%d) + C(%d) - Y(%d,%d) <= 1\n" % (i,j,i,j)  # create the needed inequality for i and j
                                                                   # so if C(i) and C(j) are both set to 1,
                                                                   # variable Y(i,j) will also be set to 1.

          Yconstraints = Yconstraints + Ystring   # concatenate that inequality to the variable 'Yconstraints'

          Ystring = "2 Y(%d,%d) - C(%d) - C(%d) <= 0\n" % (i,j,i,j)  # create the needed inequality for i and j
                                                                     # so variable Y(i,j) can be set to 1 only if
                                                                     # both C(i) and C(j) are set to 1

          Yconstraints = Yconstraints + Ystring   # concatenate that inequality to the variable 'Yconstraints'

          listY = listY + "- %f Y(%d,%d) " % (p,i,j)   # concatenate the string "+ Y(i,j) " to the variable 'listY'
          Ybinaries = Ybinaries + "Y(%d,%d) \n" % (i,j) # and concatenate "+ Y(i,j) \n" to the variable 'Ybinaries'

Sconstraint = "NumV " + listY + "<= 0 \n"   # Sconstraint holds the inequality that relates the number of old edges in the
                                          # near-clique to the number of new edges, i.e., bounding the number of new edges
                                          # to be at most p times the number of old edges.

INFILE.close()
OUTFILE.write("Maximize \n") # write to file the string (word) 'Maximize' and a new line command ('\n')
OUTFILE.write(listC + "\n")  # write to file the value of variable 'listC' and move to a new line
OUTFILE.write (Vconstraints)  # write to file the value of variable 'Vconstraints'
OUTFILE.write (Yconstraints)  # write to file the value of variable 'Yconstraints'
OUTFILE.write (Sconstraint)  # write to file the value of variable 'Sconstraint'
OUTFILE.write (listV + " - NumV  = 0 \n") # this will set the ILP variable NumV to be the number of old edges used, which allows us
                                      # to see that number easily in the .sol file.
OUTFILE.write ("binary \n")  # write to file the string 'binary' and a new line command
OUTFILE.write (Cbinaries)    # write to file the value of the variable 'Cbinaries'
OUTFILE.write (Vbinaries)    # write to file the value of the variable 'Vbinaries'
OUTFILE.write (Ybinaries)    # write to file the value of the variable 'Ybinaries'
OUTFILE.write ("end")        # write to file the string (word) 'end'
OUTFILE.close()
print ("The ILP file is %s \n" % arg2)
