# Nclique.py written for Python 2.7
# DG October 18, 2017
#
# This Python program reads a file describing a graph 
# and creates the concrete ILP for the maximum Near-clique problem,
# for that graph. The definition of a near-clique in this program
# is a subset of nodes that is either a true clique, or would be
# a true clique with the addition of one more edge.

# Call on a command line in a terminal window as:
# python Nclique.py graph-file ILP-file.lp
#

# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

arg1 = sys.argv[1]    # assign the first argument to the variable arg1
arg2 = sys.argv[2]    # assign the second argument to the variable arg2
                      # The values in arg1 and arg2 are strings.

INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read from the file.
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write to the file.

constraints = "such that \n\n"  # assign the string `such that \n\n' to the variable `constraints'
listC = ""                      # assign the empty string to the variable `listC'
listV = ""                      # assign the empty string to the variable `listV' 
Cbinaries = ""                  # assign the empty string to the variable `Cbinaries' 
Vbinaries = ""                  # assign the empty string to the variable `Vbinaries'

i = 0
for line in INFILE:  # read in the lines from INFILE, one line at a time
    i = i + 1   # increment i to keep track of the line number just read in.
    listC = listC + "+ C(%d) " % i   # concatenate the string "+ C(i) " to the variable 'listC'
    Cbinaries = Cbinaries + "C(%d)\n" % i # and concatenate "C(i) \n" to the variable 'Cbinaries'

    j = 0    
    for char in line:   # for each character in the line just read in:
       j = j + 1    # increment j to keep track of the character position being examined.
       if char == '0' and (i < j):  # if the char is a '0', create the next string. But we don't
                                    # want to generate the same inequality twice, so we only
                                    # do it if i < j.
          string = "C(%d) + C(%d) - V(%d,%d) <= 1\n" % (i,j,i,j)  # create the needed inequality for i and j
                                                                  # so if C(i) and C(j) are both set to 1,
                                                                  # variable V(i,j) will also be set to 1.
          constraints = constraints + string   # concatenate that inequality to the variable 'constraints'

          listV = listV + "+ V(%d,%d) " % (i,j)   # concatenate the string "+ V(i,j) " to the variable 'listV'
          Vbinaries = Vbinaries + "V(%d,%d) \n" % (i,j) # and concatenate "+ V(i,j) \n" to the variable 'Vbinaries'

constraints = constraints + listV + "<= 1 \n"  # concatenate the contents of variable `listV' to the existing constraints,
                                               # and then concatenate the string "<= 1 \n" to the constraints.
                                               # Change the constant (now 1) to allow more than one new edge.

INFILE.close()
OUTFILE.write("Maximize \n") # write to file the string (word) 'Maximize' and a new line command ('\n')
OUTFILE.write(listC + "\n")  # write to file the value of variable 'listC' and move to a new line
OUTFILE.write (constraints)  # write to file the value of variable 'constraints'
OUTFILE.write ("binary \n")  # write to file the string 'binary' and a new line command
OUTFILE.write (Cbinaries)    # write to file the value of the variable 'Cbinaries'
OUTFILE.write (Vbinaries)    # write to file the value of the variable 'Vbinaries'
OUTFILE.write ("end")        # write to file the string (word) 'end'
OUTFILE.close()

print ("The ILP file is %s \n"  % arg2)
