# assignment.pl
#
# June 12, 2016 
# DG
# modified from  graphcTSPfile.pl 
# to read in a weighted undirected graph from file,
# and create just the assignment inequalities.
#
# Execute on a command line in a terminal window:
#
# perl assignment.pl name-of-edge-weight-file upper-bound-on-real-weight
#
open (IN, "$ARGV[0]");  # name of the file holding the input edge-weight matrix.
open (TSP, ">A$ARGV[0].lp");  # the output file prepends a A and appends .lp to the name of the input file
open (TRACE, ">trace");
$bound = $ARGV[1]; # upper bound on meaningful data. Values larger that this bound 
                   # indicate that the edge does not exist.
@D = ();
%bin = ();
%generals = ();
$F0 = "";
$source_constraints = "";
$sink_constraints = "";
$fconstraints = "";
$fflow = "";
$infflow = "";
$outfflow = "";

$i = 1;
while ($line = <IN>) {
    chomp $line;
    $linelen = length($line) - 1;
    @linedata = split (/ /, $line);
       foreach $j (1 .. $linelen) {
             $D[$i][$j] = $linedata[$j-1];
       }
    $i++;
}
$n = $i-1; # this is the number of rows in the input matrix for the graph.

foreach $p (1..$n-1) {   
    foreach $q ($p+1..$n) {

       if ($D[$p][$q] <= $bound) {   # check if the distnace is from a real edge.

            $objective .= "+ $D[$p][$q] F$p,$q "; # because the edges are undirected, we need both terms
            $objective .= "+ $D[$p][$q] F$q,$p ";
            print TRACE "XX $objective \n";
            $constraints .= "F$p,$q + F$q,$p <= 1\n";
            $bin{"F$p,$q"} = 1;
            $bin{"F$q,$p"} = 1;
        }
        else {
            $F0 .= "F$p,$q = 0 \n";
            $F0 .= "F$q,$p = 0 \n";

        }

    } # end of foreach q
}

print TSP "minimize\n";
print TSP "$objective\n";
print TSP "such that\n";

foreach $i (1..$n){      # here we set up the equalities that say that there must be an entrance to, and 
                         # an exit from each city.
                         #
   $Fstring = "";  # used to force one exit from each node $i.
   $Istring = "";  # used to force one entrence to each node $i.

   foreach $p (1..$n){
        $index = "$i,$p";
        if (($i != $p) and ($bin{"F$index"} == 1)) {
           $Fstring .= "+ F$index ";  # this ensures that we have a tour, not just a path. DG??
           $Istring .= "+ F$p,$i ";
        }
   }
      print TSP "$Fstring = 1\n";
      print TSP "$Istring = 1\n";
      print TSP  "\n";
}



print TSP "$constraints \n";
print TSP "$F0";
print TSP "binaries\n";
foreach $var (sort keys %bin) {
  print TSP "$var\n";
}

print TSP "end";
close (TSP);
