#!/pkg/bin/perl -w
#charcliquefast.pl
# Jan 9, 2016
# Dan Gusfield
#
# This program creates the concrete ILP formulation to solve
# the Maximum Character-Clique problem (CCP). If the optimal solution
# has size k, and the input matrix has  n sites, then the optimal
# solution to the minimum character removal (MCR) problem has size
# n - k. Further, the sites in the two optimal solutions are complements of each other.

# Call the program as:
# perl charcliquefast.pl data-file
#
# Example data file: char-data
#
# The ILP formulation will be in a file with the same name as the data-file, but with the ".lp" extension.
#
# The solution to the ILP formulation sets variable Xi to 1 to mean that column i is part of a largest
# clique of pairwise compatible columns. So conversely, the columns associated with the X variables with value 0
# in the optimal solution, form a solution to the MCR problem.
#
# use diagnostics
#
open IN, "$ARGV[0]";
open OUT, ">$ARGV[0].lp";
@lines = <IN>;  # The format does not expect any lines before the data 
$line = $lines[1];
$line =~ tr/ //d;
#print "$line";
chomp $line;
$stringlen = length ($line);
#print "$stringlen ***\n";

   %count = ();


$left = 0;
$right = $stringlen;

   foreach $i ($left...$right -2) {
     foreach $j ($i+1...$right -1) {
     $ii = $i+1;
     $jj = $j+1;
     $key = $ii . ',' . $jj;
     %{$hash{$key}} = ();     
     $count{$key} = 0;
     }
   }
     
  print OUT "Maximize\n";
  foreach $i (1 .. $stringlen) {
     print OUT "+ X$i\n";
  }
  print OUT "subject to\n";

foreach $line (@lines) {
 chomp $line;
 $line =~ tr/ //d;
   @chars = split(//, $line);
   foreach $i ($left...$right -2) {
     foreach $j ($i+1...$right -1) {
     $ii = $i+1;
     $jj = $j+1;
     $key = $ii . ',' . $jj;
     $value  = $chars[$i] . ',' . $chars[$j];
     if (! defined $hash{$key}{$value}) {
       $hash{$key}{$value} = 1;
       $count{$key}++;
   # print OUT "Another $key; $value, $count{$key}\n";
     }
     }
  } 
}

 $count = $conflictcount = 0;
 foreach $ekey (sort keys %count) {
  if ($count{$ekey} == 4) {
  ($ii, $jj) = split(/,/, $ekey);
#  print OUT "$ii, $jj **\n";
  print OUT "X$ii + X$jj <= 1 \n";
  $count++;
    if (! defined $conflict{$ii}) {
       $conflict{$ii} = 1;
       $conflictcount++;
    }
    else {
       $conflict{$ii}++;
    }
    if (! defined $conflict{$jj}) {
       $conflict{$jj} = 1;
       $conflictcount++;
    }
    else {
       $conflict{$jj}++;
    }
  }
 }
#if ($conflictcount > 0) {
# $conflictratio = $count/$conflictcount;
# print OUT "The number of sites that are involved in a conflict is $conflictcount \n
# the ratio of pairs to sites is $conflictratio. \n
# The conflicted sites are: \n";
#}

# foreach $i (sort {$a <=> $b} keys %conflict) {
#  print OUT "$i $conflict{$i}\n";
# }

#  print OUT "\n\n";
 $totval = 0;
 foreach $val (sort {$b <=> $a} values %conflict) {
  $totval += $val;
#  print OUT "$val, $totval\n";
 }

  print OUT "\nbinary\n";
  foreach $i (1 .. $stringlen) {
     print OUT "X$i\n";
  }
  print OUT "end\n";

print "The ILP formulation is in file $ARGV[0].lp \n";
close (IN);
close (OUT);
