# inverseclique.py written for Python 2.7
#
# DG October 24, 2017
#
# This Python program reads a file describing a graph, and the size, denoted k, of 
# the largest clique  in the graph (previously determined); and the size, d, of the required increment.
# The program creates the concrete ILP for the inverse Near-clique problem,
# for that graph. The ILP, when executed, finds the minimum number of new edges that have to
# be added so that the resulting graph has a clique of size k + d.
#
# Call on a command line in a terminal window as:

# python inverseclique.py graph-file ILP-file.lp k d
# Example Data: example-graph 
# which has a clique of size k = 8. 

# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

arg1 = sys.argv[1]    # assign the first argument to the variable arg1
arg2 = sys.argv[2]    # assign the second argument to the variable arg2
                      # The values in arg1 and arg2 are strings.

k = sys.argv[3]       # assign the third argument to variable k. k holds the size of the previously determined max-clique in the input graph  
d = sys.argv[4]       # assign the fourth argument to variable d. d holds the required increment for the max-clique size
k = int(k)            # convert k from a string variable to an integer variable
d = int(d)            # convert d from a string variable to an integer variable

INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read from file the matrix description of the input grah.
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write to the ILP formulation to file.

constraints = ""                # assign the empty string to the variable `constraints'
listC = ""                      # assign the empty string to the variable `listC'
listV = ""                      # assign the empty string to the variable `listV' 
Cbinaries = ""                  # assign the empty string to the variable `Cbinaries'
Vbinaries = ""                  # assign the empty string to the variable `Vbinaries'

i = 0
for line in INFILE:  # read in the lines from INFILE, one line at a time
    i = i + 1   # increment i to keep track of the line number just read in.
    listC = listC + "+ C(%d) " % i   # concatenate the string "+ C(i) " to the variable 'listC'
    Cbinaries = Cbinaries + "C(%d)\n" % i # and concatenate "C(i) \n" to the variable 'Cbinaries'

    j = 0    
    for char in line:   # for each character in the line just read in:
       j = j + 1    # increment j to keep track of the character position being examined.
       if char == '0' and (i < j):  # if the char is a '0', create the next string. But we don't
                                    # want to generate the same inequality twice, so we only
                                    # do it if i < j.
          string = "C(%d) + C(%d) - V(%d,%d) <= 1\n" % (i,j,i,j)  # create the needed inequality for i and j
                                                                  # so if C(i) and C(j) are both set to 1,
                                                                  # variable V(i,j) will also be set to 1.
          constraints = constraints + string   # concatenate that inequality to the variable 'constraints'

          listV = listV + "+ V(%d,%d) " % (i,j)   # concatenate the string "+ V(i,j) " to the variable 'listV'
          Vbinaries = Vbinaries + "V(%d,%d) \n" % (i,j) # and concatenate "+ V(i,j) \n" to the variable 'Vbinaries'

constraints = constraints + listV + "- V <= 0\n"

INFILE.close()
OUTFILE.write("Minimize V \n") # write to file the string 'Minimize V' and a new line command ('\n')
OUTFILE.write("Such that \n")
target = k + d
nextstring = listC + ">= %d \n" % target # write to file the value of variable 'listC' then ">= target" and move to a new line

OUTFILE.write (nextstring)  # write to file the value of variable 'nextstring'
OUTFILE.write (constraints)  # write to file the value of variable 'constraints'
OUTFILE.write ("binary \n")  # write to file the string 'binary' and a new line command
OUTFILE.write (Cbinaries)    # write to file the value of the variable 'Cbinaries'
OUTFILE.write (Vbinaries)    # write to file the value of the variable 'Vbinaries'
OUTFILE.write ("end")        # write to file the string (word) 'end'
OUTFILE.close()
