# kcutf.py  written for Python 2.7
#
# updated November 26, 2017
# October 18, 2017
# DG
#
# This Python program reads a file describing a graph, and another file
# specifying the features of some of the nodes.
# Then it creates the concrete ILP for the min kcut problem, i.e., the cut that separates
# the nodes of different given features, minimizing the number of edges that cross the cut.

# The program can be used to find an (s,t) min-cut, just by specifying only two nodes, one with feature
# 1, and the other with feature 2.
#
#
# Call the program on a command line in a terminal window as:
# python kcutf.py graph-file-name feature-file-name ilp-file-name.lp number-of-nodes

# graph-file-name is the name of the file that holds the adjacency matrix for the graph. Example: zacharymatrix.txt
# feature-file-name is the name of the file that specifies which nodes have given features, and what those feature are. Example: zachfeats
# gives the features 1 through 4, for four nodes. Each line specifies a node and a feature number.
# ilp-file-name is the name of the file holding the concrete ILP formulation that computes an optimal partition for the specified graph
# and the specified features.
# number-of-nodes specifies the number of nodes in the input graph



import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

graph = sys.argv[1]    # assign the first argument to the variable arg1
features = sys.argv[2]    # assign the second argument to the variable arg2
                      # The values in arg1 and arg2 are strings.

ilp = sys.argv[3] # the name of the output ilp file

objective = ""
constraints = ""
binaries = ""

INGRAPH = open(graph, "r")  # open the file specified by the value of arg1, to read from the file.
INFEATURES = open(features, "r") # open the file specified by the value of arg2, to read the features from a file
OUTILP = open(ilp, "w")

n = sys.argv[4]  # the number of nodes in the input graph
n = int(n)

np1 = n+1
edgecount = 0

# in the next block, we read in the nodes that have features, and what the features are. Be sure that the last
# entry has the largest feature number. It is suggested that the (node, feature) entries be sorted in order of
# feature number, smallest first

for line in INFEATURES:
#      print (line)
      (s,f) = line.split()
      s = int(s)    
      f = int(f)
      maxf = f
#      print "%d %d" % (s,f)
      constraints = constraints + "X(%d,%d) = 1 \n" % (s, f) # X(s, f) # records that node i has feature f
      binaries = binaries + "X(%d,%d) \n" % (s, f)

i = 0
for line in INGRAPH:
     i = i + 1
     for f in range(1, maxf+1):
         constraints = constraints + "+ X(%d,%d) " % (i,f)  # set up equality to ensure that node i is assigned a feature during
                                                            # the ILP execution.
         binaries = binaries + "X(%d,%d) \n" % (i, f)

     constraints = constraints + "= 1 \n"                   # these constraints are redundant for the nodes that were 
                                                            # assigned a feature from INFEATURES, but Gurobi preprocesser will
                                                            # sort that out.

     j = 0
     for char in line:
         j = j + 1
         if (j > i):
           if (char == '1'):                 # (i,j) is an edge
                  edgecount = edgecount + 1 
                  objective = objective + "+ Z(%d,%d) " % (i,j)
                  binaries = binaries + "Z(%d,%d) \n" % (i,j)
                  for f in range(1,maxf+1):
                      constraints = constraints + "Z(%d,%d) + X(%d,%d) - X(%d,%d) >= 0 \n" % (i,j,i,f,j,f)
                      constraints = constraints + "Z(%d,%d) - X(%d,%d) + X(%d,%d) >= 0 \n" % (i,j,i,f,j,f)


#end of block 1
OUTILP.write ("Minimize\n")
OUTILP.write (objective + "\n")
OUTILP.write ("such that \n\n")
OUTILP.write(constraints)
OUTILP.write("\nbinary \n")
OUTILP.write(binaries)
print "The ILP file is %s \n" %   ilp
