# largest-dense.pl
#
# creates the ILP formulation to find a 
# largest subgraph (number of nodes) that has density at least that of the input value.
# DG April 7, 2018

# call on a command line in a terminal file as:
# perl largest-dense.pl graph-file-name density

open (IN, $ARGV[0]);   # the input file where the input graph adjacency matrix is specified.
open (OUT, ">$ARGV[0].lp");
$ilpfile = "$ARGV[0].lp";
$density = $ARGV[1]; # a number between 0 and 1

print OUT "Maximize\n";

$objective = "";
$constraints = "";
$binaries = "";
$nodenum = 0;

$potentials = "";
$edges = "";
$binaries = "";

$i = 1;
while ($line = <IN>) {
   chomp $line;
   $nodenum = length($line);
   @chars = split (//, $line);

   $objective .=  "+ C$i ";
   $binaries .= "C$i\n";

  foreach $j ($i+1 .. $nodenum) {
   
    $constraints .= "C$i + C$j - P$i,$j <= 1\n";  # this records that there is a potential edge 
                                                  # between $i and $j, if both $i and $j are chosen.

#    $constraints .= "2 P$i,$j - C$i - C$j <= 0\n"; # this imposes that a potential edge between $i and $j is recorded only
                                                   # if both $i and $j are taken.

    $potentials .= "- $density P$i,$j ";

    $binaries .= "C$i\nC$j\n";
    $binaries .= "P$i,$j\n";

    if ($chars[$j-1] eq 1 ) { # when this holds, there is an edge in the graph between nodes $i and $j
      $constraints .= "2 E$i,$j - C$i - C$j <= 0\n";  # this says that if E$i,$j is set to 1, there must be an edge
                                                    # between $i and $j, and both those nodes have been chosen.

#      $constraints .= "C$i + C$j - E$i,$j <= 1\n";  # this records that there is a real edge between $i and $j$ in the chosen subgraph, if both
                                                    #  $i and $j are taken. 

      $edges .= "+ E$i,$j ";
      $binaries .= "E$i,$j\n";
    }
   }
$i++;
} # end while

print OUT "$objective \n";
print OUT "subject to \n";
print OUT "$constraints \n"; 
print OUT "$edges \n $potentials >= 0 \n";
print OUT "$edges - NumE = 0 \n";   # NumE gets set to the sum in $edges, so that after Gurobi finds
                                    # an optimal solution, the value of NumE will be reported in the .sol file.
print OUT "general\n";
print OUT "NumE \n";
print OUT "binary\n";
print OUT "$binaries";
close OUT;

print "The ILP formulation is in file $ilpfile \n";
