#marker-pipeline.pl
# DG
# August 17, 2017
#
use Benchmark;
use Time::localtime; 

# This pipeline, written in Perl, generates marker-probe data for the Karp et al. physical mapping problem
# discussed in Chapter 9.
# It generates several blocks of consecutive ones per row; and
# then generates the GG compact ILP formulation to solve it;
# calls gurobi to optimize it; and then extracts and displays the results.
#
# On a command line, call the program as:
# perl marker-pipeline.pl name-for-new-data-file number-of-rows number-of-cols max-block-length

$rep = 1; #  $rep is the number of datasets to test. Set the number with the right-hand-side value.
foreach $i (1 .. $rep) {  

$newdata = $ARGV[0];
$rows = $ARGV[1];
$sites = $ARGV[2];
$maxrun = $ARGV[3];
open (OUTLOG, '>>outlog'); # keep a log of the computation, to help fix any errors in the program

print OUTLOG "\n\n New multiple blocks Test $rows, $sites, $maxrun \n";

system ("cp gurobi.log gurobi.sav"); # save the Gurobi log 
system ("rm gurobi.log");
system ("perl markerTSP.pl $newdata $rows $sites $maxrun ");  # create the random data
$distfile = $newdata . 'dist';
$datafile = $newdata . 'data';
print "The distance matrix is in file $distfile \n";

print OUTLOG "Generating the GG formulation\n";
#system ("perl MgraphTSPfile.pl $distfile 200 ");  # this is for TS path with no column permutations 
#system ("perl MgraphTSPfile.pl p$distfile 200 "); # this is for TS path after permuting the columns
#system ("perl M1graphTSPfile.pl $distfile 200 ");  # Jan. 14, 2017, this is for TS tour with no column permutations
system ("perl M1graphTSPfile.pl p$distfile 200 ");  # this is for TS tour after permuting the columns

#system ("rm M$distfile.sol");
#system ("rm cM$distfile.sol");


print "\n\n **** Starting Gurobi - Ignore Unknown Variable Messages - they are caused by programmer (me) laziness.\n\n";
#system ("gurobi_cl resultfile=M$distfile.sol M$distfile.lp"); # for unpermuted data, path problem
#system ("gurobi_cl resultfile=Mp$distfile.sol Mp$distfile.lp");  # for permuted data, path problem

#system ("gurobi_cl resultfile=M$distfile.sol M1$distfile.lp"); # for unpermuted data, tour problem
system ("gurobi_cl resultfile=Mp$distfile.sol M1p$distfile.lp");  # for permuted data, tour problem

#system ("perl cleanres.pl M$distfile.sol");  # for unpermuted data, path problem
#system ("perl cleanres.pl Mp$distfile.sol");  # for permuted data, path problem

#system ("perl cleanres.pl M$distfile.sol");  # for unpermuted data, tour problem
system ("perl cleanres.pl Mp$distfile.sol");  # for permuted data, tour problem

print "The datafile is $datafile \n";
#system ("perl markersolcheck.pl cM$distfile.sol $sites $datafile"); # compare the solution found with the original, for unpermuted data.
system ("perl pmarkersolcheck.pl cMp$distfile.sol $sites $datafile p$datafile");  # compare for permuted data

}
close (OUTLOG);
