# markerTSP.pl 
# DG August 16, 2017
#
# Creates a dataset mimicing what one could get from clone-marker data with multiple
# fragments. Output the dataset, and both the distance matrix derived from the dataset
# and the distance matrix derived from the matrix with the columns permuted.
# 
# Call the program on a command line in a terminal window as:
# perl markerTSP.pl  number-of-rows number-of-cols max-block-length
#
open (OUT, ">$ARGV[0]data"); # output file for the unpermuted data
open (OUTp, ">p$ARGV[0]data"); # output file for the column-permuted data
open (OUT1, ">$ARGV[0]dist"); # distances derived from the unpermuted data
open (OUT2, ">p$ARGV[0]dist"); # distances derived from a permutation of the cols.
$n = $ARGV[1]; # number of rows
$nm1 = $n - 1;

$sites = $ARGV[2]; # number of sites
$maxrun = $ARGV[3]; # maximum length of an interval.  
@data = ();
@pdata = ();
@s = ();
@D = ();
@PD = ();
%bin = ();
%generals = ();


$totalblocks = 0; # totalblocks will about twice the true number of blocks

foreach $i (1..$n) {  # generate n rows, each generated with intervals of length between 4 and maxrun

   $rightmark = 1;
   $runflag = int (rand(2)) ; # runflag is used to tell whether the next interval will be 1s or 0s
                              # it starts out random.

   while ($rightmark <= $sites) {
       $increment = int (rand($maxrun) + 3);  # randomly choose an increment.
       $next = $rightmark + $increment;

       foreach $j ($rightmark  .. $next) { # 
                $data[$i][$j] = $runflag;
                }
       if ($runflag == 1) {
            $totalblocks++;
       }
       $runflag = 1 - $runflag;
       $rightmark = $next + 1;
    } 

   foreach $j (1 .. $sites) {
       print OUT "$data[$i][$j]";
   }
   print OUT "\n";
}
   print "$totalblocks \n";

# create a permutation of integers 1 through $sites

foreach $i (1 .. $sites) {
     $s[$i] = $i;
}

foreach $i (1 .. $sites) {
  $permi = int rand($sites) + 1;  # generate a random integer between 1 and $sites
  $sav = $s[$permi]; # exchange the entries in site $i of s with the entry in site $permi in $s
  $s[$permi] = $s[$i];
  $s[$i] = $sav;
}


#foreach $i (1 .. $sites) {  # print the permutation
#   $index = $s[$i];
#   print "$index ";
#}
print "\n";

# Using the permutation, create the permuted data matrix pdata; 

print "The permuted data matrix \n";
foreach $i (1 .. $n) {
  foreach $j (1 .. $sites) {
      $pdata[$i][$j] = $data[$i][$s[$j]];
      print "$pdata[$i][$j]"; 
      print OUTp "$pdata[$i][$j]"; 
  }
  print "\n";
  print OUTp "\n";
}

###  now compute the distances

foreach $i (0 .. $sites) {
   foreach $j (0 .. $sites) {
      $D[$i][$j] = 0;   # the distance matrix using the matrix data
      $PD[$i][$j] = 0;  # this is the distance matrix using the permuted matrix
   }
}


foreach $j (1 .. $sites) {  # count the number of 1s in each column
   $num1s = 0;
   $pnum1s = 0;
   foreach  $i (1 .. $n) {
      if ($data[$i][$j] == 1) {
         $num1s++;
      }

      if ($pdata[$i][$j] == 1) {
         $pnum1s++;
      }
    }
#   print OUT "$j: $num1s \n";
   $D[$j][0] = $D[0][$j] = $num1s;
   $PD[$j][0] = $PD[0][$j] = $pnum1s;
}

foreach $p (1 .. $sites - 1) {
   foreach $q ($p + 1 .. $sites) {
     $Hamm = 0;
     $pHamm = 0;
     foreach $i (1 .. $n) {
        if ($data[$i][$p] != $data[$i][$q]) {
           $Hamm++;
        }
        if ($pdata[$i][$p] != $pdata[$i][$q]) {
           $pHamm++;
        }
     }
   $D[$p][$q] = $Hamm;
   $D[$q][$p] = $Hamm;
   $PD[$p][$q] = $pHamm;
   $PD[$q][$p] = $pHamm;
   } 
}


foreach $i (0 .. $sites) {
#   print OUT1 "$i: ";
   foreach $j (0 .. $sites) {
     print OUT1 "$D[$i][$j] ";
     print OUT2 "$PD[$i][$j] ";
   }
   print OUT1 "\n";
   print OUT2 "\n";
}

close(IN);
close(OUT);
close(OUTp);
close(OUT1);
close(OUT2);
