# modular2.py written for Python 2.7
# DG
#
# This Python program reads a file describing a graph, and produces an ILP
# formulation to compute modularity and produce the node partition of the graph
# that defines the modularity, as defined by GN. 

# call on a command line in a terminal window as:
# python modular2.py input-file output-file.lp number-of-nodes
#
# input example: zacharymatrix.txt

# The code here reverses the meaning of the X variables, and adjusts
# the objective function accordingly. That is X(i,j) is set to one if nodes i and j
# are in the same class.
#
# update Nov. 17, 2017
# DG November 12, 2017

# Note, for anyone new to programming: This looks very cluttered, but
# everything to the right of the hash mark (#) on any line (such as this one) is a comment
# ignored by python. It is to help humans understand the program.


import sys  # this imports the module called `sys', which is a collection of programs
            # that facilitate system calls

import random

arg1 = sys.argv[1]    # assign the first argument to the variable arg1
arg2 = sys.argv[2]    # assign the second argument to the variable arg2
                      # The values in arg1 and arg2 are strings.

objective = ""
constraints = ""
binaries = ""
modtotal = 0


INFILE = open(arg1, "r")  # open the file specified by the value of arg1, to read from the file.
OUTFILE = open(arg2, "w") # open the file specified by the value of arg2, to write to the file.
n = sys.argv[3]  # input how many nodes
n = int(n)
np1 = n+1

totaldegree = 0
degree = [0 for x in range(0,np1)]
adjacency = [[0 for x in range(0, np1)] for y in range(0, np1)] # initialize the adjacency matrix with all zero entries

i = 0
for line in INFILE:
     line.rstrip()
     i = i + 1
     j = 0
     for char in line:
         j = j + 1
#         print ("%d, %d" % (i,j))
         if (char == '1'):  # (i,j) is an edge

              degree[i] = degree[i] + 1
              totaldegree = totaldegree + 1
              adjacency[i][j] = 1

print (totaldegree)
m = totaldegree/2
print ("the number of edges is %d \n" % m)

# print (degree)
# print (adjacency)


# now compute the modular matrix
modular = [[0 for x in range(0, np1)] for y in range(0, np1)] # initialize the modular matrix with all zero entries

for i in range(1,n):
    for j in range(i+1, np1):
        modular[i][j] = (adjacency[i][j] - (degree[i] * degree[j])/float(2 * m)) 
        modtotal = modtotal + modular[i][j]
# print(modtotal)

for i in range(1,n):
    for j in range(i+1, np1):
        binaries = binaries + "X(%d,%d) \n" % (i,j)
        modcoifficient =  modular[i][j]
        objective = objective + "+  %s  X(%d,%d) " % (modcoifficient,i,j)  # Nov. 17, change from - to +
        for k in range(1, np1):
            if ((k != i) and (k != j)):
                constraints = constraints + "X(%d,%d) - X(%d,%d) = 0 \n" % (i,j,j,i) # require symmetry

for i in range(1,np1):
    for j in range(1, np1):
        for k in range(1, np1):
            if ((k != i) and (k != j) and (i != j)):
                constraints = constraints + "X(%d,%d) + X(%d,%d) - X(%d,%d) <= 1 \n" % (i,j,i,k,j,k)  # assure transitivity

OUTFILE.write ("Maximize \n")
OUTFILE.write (objective)
OUTFILE.write ("such that \n")
OUTFILE.write (constraints)

OUTFILE.write ("binary\n")
OUTFILE.write (binaries)
