# tangILP.pl
# Perl program to create a concrete ILP formulation to solve the tanglegrams problem, given descriptions of the two trees in
# Newick format. The input file has three lines. The first line holds a single number, which is the number of leaves
# in each tree. The second and third lines describe two trees in Newick format. Call the program as:
#
# perl tangILP.pl file-name
#
# A test case for the program is in file: tangtest

# The variable X is set to the number of tree exchanges that are made in the optimal solution. It is not used in this program, but is used
# in others. Its value is reported in a .sol file, if one is created, when the concrete ILP is executed.
#
# Due to my programming laziness, this program generates an R1 and an R2 variable for each possible interval of leaves, and puts
# these in the list of binary variables. But most of these variables 
# are not used, so Gurobi warns that these variables are unknown in the binary section. Not a serious thing -- ignore it.
# DG
# April 7, 2016

use diagnostics

open IN, $ARGV[0];
open OUT, ">$ARGV[0].lp";
@binaries = ();
%Xhash = ();

$n = <IN>;
chomp $n;

print OUT "Minimize \n C ";  # we use the variable C in the objective function. C is set later to the sum of all the C(i,j) variables.
print OUT "\n subject to \n";

foreach $i (1 .. $n-1) {
	foreach $j ($i+1 .. $n) {
		print OUT "+ C($i,$j) ";
                push (@binaries, "C($i,$j)");
        }
}
print OUT "\n -C = 0 \n";

foreach $i (1 .. $n-1) {
   foreach $j ($i+1 .. $n) {
	print OUT "C($i,$j) - R1($i,$j) + R2($i,$j) >= 0 \n";
	print OUT "C($i,$j) - R2($i,$j) + R1($i,$j) >= 0 \n";
   }
}


foreach $loop (1 .. 2) {

@stack = ();
$tree= <IN>;
chomp $tree;
# print "$n, $tree\n";
%lp = ();
$rp = ();

$len = length $tree;
# print "$len \n";

@t1 = ();
@t1 = splitwell ($tree);  # split the characters, but group consecutive digits into a single number
# print @t1;
# print  "\n";
$len = @t1;
# print "$len \n";

foreach $i (0..$len-1) {
  if ($t1[$i] eq '(' ) {
     push (@stack, $i);
  }
     
  if ($t1[$i] eq ')' ) {

     @rlist = ();
     @llist = ();
     $j =   pop (@stack);

     # print "NODE $j, $i\n";
     $lp{$i} = $j;
     $rp{$j} = $i;

     $im1 = $i - 1;
     if ($t1[$im1] eq ')') {

          foreach $k ($lp{$im1} .. $im1) {
            if ($t1[$k] =~ /\d+/) {
              push (@rlist, $t1[$k]);
            }
          }
     }
     else {
          # print "loneR $t1[$im1]\n";
          push (@rlist, $t1[$im1]);
     }

     $jp1 = $j+1;
     if ($t1[$jp1] eq '(' ) {
        foreach $k ($jp1 .. $rp{$jp1}) {
           if ($t1[$k] =~ /\d+/) {
              push (@llist, $t1[$k]);
           }
        }
     }
     else {
          # print "loneL $t1[$jp1]\n";
          push (@llist, $t1[$jp1]);
     }
     $leftend = $llist[0];
     $rightend = $rlist[-1];
       

      foreach $p (@llist) {
         foreach $q (@rlist) {
#           print OUT "R$loop($p,$q) - R$loop($q,$p) = 0\n";
           push (@binaries, "R$loop($p,$q)");
           push (@binaries, "R$loop($q,$p)");
           if (! defined $Xhash{"$loop($leftend,$rightend)"}) {
                push (@binaries, "X$loop($leftend,$rightend)");
                $Xtotal .= "+ X$loop($leftend,$rightend) ";
                $Xhash{"$loop($leftend,$rightend)"} = 1; 
           }


           if ($p > $q) {
              # print "Found p > q: $p, $q, $j, $i \n";
              print OUT "R$loop($q,$p) - X$loop($leftend,$rightend) = 0 \n";
           }
           else {
              print OUT "R$loop($p,$q) + X$loop($leftend,$rightend) = 1 \n";
           }
          }
       }           
     # print "$llist[0], $rlist[-1]\n";
     # print "\n";
   } 
}

}
print OUT "$Xtotal \n - X = 0 \n";

print OUT "binary \n";

foreach $var (@binaries) {
   print OUT "$var \n";
}
 
print OUT "end\n";

print "The ILP formulation is in file: $ARGV[0].lp \n";

sub splitwell{

@NA = ();
$S = $_[0];
@SA = split (//,$S);
# print "Inside splitwell @SA \n";
$len = @SA;
# print "Inside splitwell length $len \n";

$num = "";

$working = 0;
$j = 0;
foreach $i (0 .. $len -1) {
#  print "Testing for  j num, SA: $j, $num, $SA[$i] \n";
  if ($SA[$i]  =~ /[(),]/) {
     if ( $working == 1){
       $NA[$j] = int $num;
       $num = "";
       $j++;
       $working = 0;
     }
     $NA[$j] = $SA[$i];
     $j++;
  }
  else {
    $working = 1;
    $num .= $SA[$i];
  }
}

$len = @NA;
# print "Inside splitwell NA @NA, $len\n";
return @NA;

#print "$len \n";
}
